"use strict";

var TDPAHCMSResponseObject=function TDPAHCMSResponseObjectFn(responseObject, callback)
{
	// Required libs
	var util=require("util");
	var path=require("path");

	// JSONCuraotor - the main requirement
	var TDPJSONCurator=require("tdp-json-curator");
	// for debugging...
	// var TDPJSONCurator=require(path.join(__dirname, "../TDPJSONCurator/TDPJSONCurator.js"));


	var responseObjectSpec=require(path.join(__dirname, "/config/TDPAHCMSResponseObjectSpec.js"));


	// Private vars and methods
	var constructorResponseObject=responseObject;
	var ROType=typeof(constructorResponseObject);
	var constructorCallback=callback;
	var CBType=typeof(constructorCallback);

	// Parse the supplied arguments and call init() using them (where appropriate)
	if(ROType==="function") // If only a callback function was supplied we nullify the constructorResponseObject so init() will use defaults
	{		
		constructorResponseObject=null;
		constructorCallback=responseObject;
	}
	else if(ROType!=="object" || CBType!=="function")
	{
		throw new Error("You must supply valid constructor options: [responseObject (object},] callback (function)");
	}


	var DROType=typeof(constructorResponseObject);

	// If constructor has an object passed in we'll try to use it natively
	if(DROType==="object" && constructorResponseObject!==null)
	{
		constructorResponseObject=constructorResponseObject;
	}
	// else if a string has been passed in, try to use it as an absolute or relative path to a file which must be a commonJS module with the requisite JSON as its root
	else if(DROType==="string")
	{
		constructorResponseObject=require(constructorResponseObject);
	}
	// Else we'll use the default
	else
	{
		constructorResponseObject=require(path.join(__dirname, "/config/TDPAHCMSResponseObjectDefaults.js"));
	}


	// Instantiate curator into a global-scoped, private var using the above parsed args
	var curator=new TDPJSONCurator(constructorResponseObject, responseObjectSpec, true);



// Public vars and methods
	this.get=function getFn(property, callback)
	{
		curator.get(property, function(ret)
		{
			if(typeof(callback)==="function")
			{
				callback(ret)
				return;
			}
		});
	};

	this.getObject=function getObjectFn(callback)
	{
		curator.get(null, function getObjectFnCbk(ret)
		{
			if(typeof(callback)==="function")
			{
				callback(ret);
				return;
			}
		});
	};

	this.set=function setFn(property, value, callback)
	{	
		curator.set(property, value, function(ret)
		{
			if(typeof(callback)==="function")
			{				
				callback(ret);
				return;
			}
		});
	};

	this.setObject=function setObjectFn(objectToSet, callback)
	{
		curator.curate(objectToSet, responseObjectSpec, true, function setObjectFnCbk(ret)
		{
			if(typeof(callback)==="function")
			{
				callback(ret);
				return;
			}
		});
	};


	// Note: more complex type checking is done inside init()
	if(constructorResponseObject!==undefined && constructorCallback!==undefined)
	{
		if(typeof(constructorCallback)==="function")
		{
			constructorCallback(this);
			return;
		}
	}
};

module.exports=TDPAHCMSResponseObject;







